#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['bq_schema', 'bq_schema.migration', 'bq_schema.types']

package_data = \
{'': ['*'], 'bq_schema': ['cli/*']}

install_requires = \
['google-cloud-bigquery==1.24.0', 'dacite==1.5.1']

extras_require = \
{'develop': ['invoke==1.4.1'],
 'test': ['black==20.8b1',
          'isort==5.6.4',
          'mypy==0.782',
          'pylint==2.6.0',
          'pytest==6.0.2',
          'pytest-cov==2.10.1']}

entry_points = \
{'console_scripts': ['convert-table = bq_schema.cli.convert_table:cli',
                     'migrate-tables = bq_schema.cli.migrate_tables:cli']}

setup(name='bq-schema',
      version='0.5.5',
      description='Define your BigQuery tables as dataclasses.',
      author='Limehome GmbH',
      author_email='raphael.brand@limehome.de',
      url='https://github.com/limehome/bq-schema',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.7',
     )
